@@ -2,6 +2,7 @@
  * panel-background-monitor.h:
  *
  * Copyright (C) 2001, 2002 Ian McKellar <yakk@yakk.net>
+ *                     2002 Sun Microsystems, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,11 +27,6 @@
 #ifndef __PANEL_BACKGROUND_MONITOR_H__
 #define __PANEL_BACKGROUND_MONITOR_H__
 
-/* FIXME: this needs to be made multiscreen aware
- *        panel_background_monitor_get should take
- *        a GdkScreen argument.
- */
-
 #include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -53,12 +49,12 @@
 typedef struct _PanelBackgroundMonitorClass PanelBackgroundMonitorClass;
 typedef struct _PanelBackgroundMonitor      PanelBackgroundMonitor;
 
-GType                   panel_background_monitor_get_type   (void);
-PanelBackgroundMonitor *panel_background_monitor_get        (void);
-GdkPixbuf              *panel_background_monitor_get_region (PanelBackgroundMonitor *monitor,
-							     int                     x,
-							     int                     y,
-							     int                     width,
-							     int                     height);
+GType                   panel_background_monitor_get_type       (void);
+PanelBackgroundMonitor *panel_background_monitor_get_for_screen (GdkScreen *screen);
+GdkPixbuf              *panel_background_monitor_get_region     (PanelBackgroundMonitor *monitor,
+								 int                     x,
+								 int                     y,
+								 int                     width,
+								 int                     height);
 
 #endif /* __PANEL_BACKGROUND_MONITOR_H__ */
