@@ -25,6 +25,8 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
@@ -34,6 +36,7 @@
 #include "panel-gdm.h"
 #include "panel-power-manager.h"
 #include "panel-session.h"
+#include "panel-icon-names.h"
 
 #define PANEL_LOGOUT_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_LOGOUT_DIALOG, PanelLogoutDialogPrivate))
 
@@ -135,10 +138,12 @@ panel_logout_init (PanelLogoutDialog *logout_dialog)
 	logout_dialog->priv->timeout    = 0;
 	logout_dialog->priv->default_response = GTK_RESPONSE_CANCEL;
 
-	gtk_window_set_skip_pager_hint (GTK_WINDOW (logout_dialog), TRUE);
+	/* FIXME: we should most probably use gtk_window_set_transient_for(PANEL) */
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (logout_dialog), TRUE);
 	gtk_window_set_keep_above (GTK_WINDOW (logout_dialog), TRUE);
 	gtk_window_stick (GTK_WINDOW (logout_dialog));
+	gtk_window_set_position (GTK_WINDOW (logout_dialog),
+				 GTK_WIN_POS_CENTER_ALWAYS);
 
 	logout_dialog->priv->power_manager = panel_get_power_manager ();
 
@@ -212,6 +217,7 @@ panel_logout_timeout (gpointer data)
 {
 	PanelLogoutDialog *logout_dialog;
 	char              *secondary_text;
+	char              *name;
 	int                seconds_to_show;
 
 	logout_dialog = (PanelLogoutDialog *) data;
@@ -233,15 +239,23 @@ panel_logout_timeout (gpointer data)
 
 	switch (logout_dialog->priv->type) {
 	case PANEL_LOGOUT_DIALOG_LOGOUT:
-		secondary_text = ngettext ("You will be automatically logged "
+		secondary_text = ngettext ("You are currently logged in as "
+					   "\"%s\".\n"
+					   "You will be automatically logged "
 					   "out in %d second.",
+					   "You are currently logged in as "
+					   "\"%s\".\n"
 					   "You will be automatically logged "
 					   "out in %d seconds.",
 					   seconds_to_show);
 		break;
 	case PANEL_LOGOUT_DIALOG_SHUTDOWN:
-		secondary_text = ngettext ("This system will be automatically "
+		secondary_text = ngettext ("You are currently logged in as "
+					   "\"%s\".\n"
+					   "This system will be automatically "
 					   "shut down in %d second.",
+					   "You are currently logged in as "
+					   "\"%s\".\n"
 					   "This system will be automatically "
 					   "shut down in %d seconds.",
 					   seconds_to_show);
@@ -250,13 +264,23 @@ panel_logout_timeout (gpointer data)
 		g_assert_not_reached ();
 	}
 
+	name = g_locale_to_utf8 (g_get_real_name (), -1, NULL, NULL, NULL);
+	if (!name || name[0] == '\0' || strcmp (name, "Unknown") == 0)
+		name = g_locale_to_utf8 (g_get_user_name (), -1 , NULL, NULL, NULL);
+
+	if (!name)
+		name = g_strdup (g_get_user_name ());
+
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (logout_dialog),
 						  secondary_text,
+						  name,
 						  seconds_to_show,
 						  NULL);
 
 	logout_dialog->priv->timeout--;
 
+	g_free (name);
+
 	return TRUE;
 }
 
@@ -311,7 +335,7 @@ panel_logout_new (PanelLogoutDialogType  type,
 
 	switch (type) {
 	case PANEL_LOGOUT_DIALOG_LOGOUT:
-		icon_name      = "gnome-logout";
+		icon_name      = PANEL_ICON_LOGOUT;
 		primary_text   = _("Log out of this system now?");
 		// FIXME need to verify that this response can be used
 		logout_dialog->priv->default_response = PANEL_LOGOUT_DIALOG_LOGOUT;
@@ -328,7 +352,7 @@ panel_logout_new (PanelLogoutDialogType  type,
 				       PANEL_LOGOUT_RESPONSE_LOGOUT);
 		break;
 	case PANEL_LOGOUT_DIALOG_SHUTDOWN:
-		icon_name      = "gnome-shutdown";
+		icon_name      = PANEL_ICON_SHUTDOWN;
 		primary_text   = _("Shut down this system now?");
 
 		logout_dialog->priv->default_response = PANEL_LOGOUT_RESPONSE_SHUTDOWN;
@@ -342,16 +366,19 @@ panel_logout_new (PanelLogoutDialogType  type,
 					       _("_Hibernate"),
 					       PANEL_LOGOUT_RESPONSE_STD);
 
-		//FIXME see previous FIXME
-		gtk_dialog_add_button (GTK_DIALOG (logout_dialog),
-				       _("_Restart"),
-				       PANEL_LOGOUT_RESPONSE_REBOOT);
+		if (gdm_supports_logout_action (GDM_LOGOUT_ACTION_REBOOT))
+			gtk_dialog_add_button (GTK_DIALOG (logout_dialog),
+					       _("_Restart"),
+					       PANEL_LOGOUT_RESPONSE_REBOOT);
+
 		gtk_dialog_add_button (GTK_DIALOG (logout_dialog),
 				       GTK_STOCK_CANCEL,
 				       GTK_RESPONSE_CANCEL);
-		gtk_dialog_add_button (GTK_DIALOG (logout_dialog),
-				       _("_Shut Down"),
-				       PANEL_LOGOUT_RESPONSE_SHUTDOWN);
+
+		if (gdm_supports_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN))
+			gtk_dialog_add_button (GTK_DIALOG (logout_dialog),
+					       _("_Shut Down"),
+					       PANEL_LOGOUT_RESPONSE_SHUTDOWN);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -370,4 +397,5 @@ panel_logout_new (PanelLogoutDialogType  type,
 
 	gtk_window_set_screen (GTK_WINDOW (logout_dialog), screen);
 	gtk_widget_show (GTK_WIDGET (logout_dialog));
+	gdk_window_focus (GTK_WIDGET (current_dialog)->window, activate_time);
 }
